
---
# NFC 7 I2C Click

> NFC 7 Click (I2C) is an add-on board that integrates NFC functionality in various applications. This board features the PN7160, a Near Field Communication (NFC) controller from NXP that is compliant with NFC Forum and NCI 2.0 standards. The board features an I2C communication interface, low-power consumption modes, and a new-generation RF contactless front-end design with high sensitivity and dynamic load modulation for reliable communication. It supports multiple NFC standards, including ISO/IEC 14443, ISO/IEC 15693, MIFARE, and FeliCa, and offers standalone card emulation functionality.

<p align="center">
  <img src="https://download.mikroe.com/images/click_for_ide/nfc7i2c_click.png" height=300px>
</p>

[Click Product page](https://www.mikroe.com/nfc-7-click-i2c)

---


#### Click library

- **Author**        : Stefan Filipovic
- **Date**          : Oct 2024.
- **Type**          : I2C type


# Software Support

We provide a library for the NFC 7 I2C Click
as well as a demo application (example), developed using MikroElektronika
[compilers](https://www.mikroe.com/necto-studio).
The demo can run on all the main MikroElektronika [development boards](https://www.mikroe.com/development-boards).

Package can be downloaded/installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

## Library Description

> This library contains API for NFC 7 I2C Click driver.

#### Standard key functions :

- `nfc7i2c_cfg_setup` Config Object Initialization function.
```c
void nfc7i2c_cfg_setup ( nfc7i2c_cfg_t *cfg );
```

- `nfc7i2c_init` Initialization function.
```c
err_t nfc7i2c_init ( nfc7i2c_t *ctx, nfc7i2c_cfg_t *cfg );
```

- `nfc7i2c_default_cfg` Click Default Configuration function.
```c
err_t nfc7i2c_default_cfg ( nfc7i2c_t *ctx );
```

#### Example key functions :

- `nfc7i2c_wait_discovery` This function waits until remote NFC device is discovered.
```c
err_t nfc7i2c_wait_discovery ( nfc7i2c_t *ctx, nfc7i2c_rf_intf_t *rf_intf );
```

- `nfc7i2c_presence_check` This function waits until the discovered target device is removed.
```c
err_t nfc7i2c_presence_check ( nfc7i2c_t *ctx, nfc7i2c_rf_intf_t *rf_intf );
```

- `nfc7i2c_stop_discovery` This function stops the RF discovery process.
```c
err_t nfc7i2c_stop_discovery ( nfc7i2c_t *ctx );
```

## Example Description

> This example demonstrates the use of NFC 7 I2C Click board by handling the detection
and processing of various NFC technologies and protocols, and ensuring the application
can respond to different NFC card types (A,B,F,V).

**The demo application is composed of two sections :**

### Application Init

> Initializes the driver and logger, performs the Click default configuration and reads the device firmware version.

```c

void application_init ( void )
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    nfc7i2c_cfg_t nfc7i2c_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    nfc7i2c_cfg_setup( &nfc7i2c_cfg );
    NFC7I2C_MAP_MIKROBUS( nfc7i2c_cfg, MIKROBUS_1 );
    if ( I2C_MASTER_ERROR == nfc7i2c_init( &nfc7i2c, &nfc7i2c_cfg ) ) 
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    if ( NFC7I2C_ERROR == nfc7i2c_default_cfg ( &nfc7i2c ) )
    {
        log_error( &logger, " Default configuration." );
        for ( ; ; );
    }
    log_printf( &logger, " FW version: %.2X.%.2X.%.2X\r\n", 
                ( uint16_t ) nfc7i2c.fw_version[ 0 ], 
                ( uint16_t ) nfc7i2c.fw_version[ 1 ], 
                ( uint16_t ) nfc7i2c.fw_version[ 2 ] );
    
    log_info( &logger, " Application Task " );
}

```

### Application Task

> Waits for an NFC device to be discovered, checks if it supports a known NFC technology, 
and then handles the device based on its protocol. The application continues processing
the device (reading and writing information) and waits until the card is removed.
Once the card is removed, the discovery process is restarted to detect a new NFC device.

```c
void application_task ( void )
{
    nfc7i2c_rf_intf_t rf_intf;
    log_printf( &logger, " WAITING FOR DEVICE DISCOVERY\r\n\n" );
    if ( NFC7I2C_OK == nfc7i2c_wait_discovery ( &nfc7i2c, &rf_intf ) )
    {
        if ( ( NFC7I2C_NCI_RF_TECH_PASSIVE_POLL_NFC_A == rf_intf.mode_tech ) || 
             ( NFC7I2C_NCI_RF_TECH_PASSIVE_POLL_NFC_B == rf_intf.mode_tech ) || 
             ( NFC7I2C_NCI_RF_TECH_PASSIVE_POLL_NFC_F == rf_intf.mode_tech ) || 
             ( NFC7I2C_NCI_RF_TECH_PASSIVE_POLL_15693 == rf_intf.mode_tech ) )
        {
            for ( ; ; )
            {
                nfc7i2c_display_card_info ( rf_intf );

                switch ( rf_intf.protocol )
                {
                    case NFC7I2C_NCI_RF_PROT_T2T:
                    {
                        nfc7i2c_handle_iso14443_3a ( &nfc7i2c );
                        break;
                    }
                    case NFC7I2C_NCI_RF_PROT_ISODEP:
                    {
                        nfc7i2c_handle_iso14443_4 ( &nfc7i2c );
                        break;
                    }
                    case NFC7I2C_NCI_RF_PROT_T5T:
                    {
                        nfc7i2c_handle_iso15693 ( &nfc7i2c );
                        break;
                    }
                    case NFC7I2C_NCI_RF_PROT_MIFARE:
                    {
                        nfc7i2c_handle_mifare ( &nfc7i2c );
                        break;
                    }
                    default:
                    {
                        break;
                    }
                }
                if ( !rf_intf.more_tags )
                {
                    break;
                }
                nfc7i2c_reader_act_next ( &nfc7i2c, &rf_intf );
            }
            
            nfc7i2c_presence_check ( &nfc7i2c, &rf_intf );
            log_printf ( &logger, " - CARD REMOVED\r\n\n" );

            nfc7i2c_stop_discovery ( &nfc7i2c );
            while ( NFC7I2C_OK != nfc7i2c_start_discovery ( &nfc7i2c ) );
        }
        else
        {
            log_printf ( &logger, " - WRONG DISCOVERY\r\n\n" );
        }
    }
}
```

The full application code, and ready to use projects can be installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

**Other Mikroe Libraries used in the example:**

- MikroSDK.Board
- MikroSDK.Log
- Click.NFC7I2C

**Additional notes and informations**

Depending on the development board you are using, you may need
[USB UART Click](https://www.mikroe.com/usb-uart-click),
[USB UART 2 Click](https://www.mikroe.com/usb-uart-2-click) or
[RS232 Click](https://www.mikroe.com/rs232-click) to connect to your PC, for
development systems with no UART to USB interface available on the board. UART
terminal is available in all MikroElektronika
[compilers](https://shop.mikroe.com/compilers).

---
